from datetime import datetime


class History:
    def __init__(self, experiment):
        self.experiment = experiment
        self.rows = []
        self._step_index = 0

    def update(self, client, row, step):
        """
        Update row in history
        """
        metrics = {}
        for k, v in row.items():
            if k in metrics:
                metrics[k].append({
                    'step': step,
                    'timestamp': datetime.utcnow().timestamp(),
                    'value': str(v),
                })
            else:
                metrics[k] = [{
                    'step': step,
                    'timestamp': datetime.utcnow().timestamp(),
                    'value': str(v),
                }]

        self.rows.append(row)
        client.experiment_metrics_update(self.experiment.id, metrics)



from urllib.parse import urlparse

import typer

from savvihub import Context
from savvihub.api.savvihub import SavviHubClient
from savvihub.common.constants import DATASET_SOURCE_TYPE_SAVVIHUB, DATASET_PATH_PARSE_SCHEME_GS, \
    DATASET_PATH_PARSE_SCHEME_S3

dataset_app = typer.Typer()


def parse_dataset_arg(dataset_arg):
    if "/" not in dataset_arg:
        typer.echo("You should specify dataset with workspace. ex) savvihub/mnist")
        return
    workspace, rest = dataset_arg.split("/")
    if ":" in rest:
        dataset, ref = rest.split(":")
    else:
        dataset = rest
        ref = "latest"
    return workspace, dataset, ref


@dataset_app.callback()
def main():
    """
    Manage the collection of data
    """
    return


@dataset_app.command()
def create(
        dataset_arg: str = typer.Argument(...),
        path_arg: str = typer.Option(None, "-u", "--url", help="Dataset source path"),
        description: str = typer.Option(None, "-m", help="Dataset description"),
        aws_role_arn: str = typer.Option(None, "--aws-role-arn", help="AWS Role ARN")
):
    """
    Create a SavviHub dataset
    """
    workspace, dataset, _ = parse_dataset_arg(dataset_arg)

    context = Context(login_required=True)
    client = SavviHubClient(token=context.token)

    if path_arg:
        if not (path_arg.startswith("gs://") or path_arg.startswith("s3://")):
            typer.echo(f"path should start with \"gs://\" or \"s3://\"")
            return

        r = urlparse(path_arg)
        if r.scheme == DATASET_PATH_PARSE_SCHEME_GS:
            dataset_obj = client.dataset_gs_create(workspace, dataset, False, description, path_arg)
        elif r.scheme == DATASET_PATH_PARSE_SCHEME_S3:
            if not aws_role_arn:
                typer.echo("AWS Role ARN is required for S3 users")
                return
            dataset_obj = client.dataset_s3_create(workspace, dataset, False, description, path_arg, aws_role_arn)
        else:
            raise Exception("Only Google Cloud Storage and Amazon S3 are supported at the moment.")
    else:
        dataset_obj = client.dataset_create(workspace, dataset, False, description)

    if not dataset_obj:
        return

    dataset_slug = dataset_obj.slug
    typer.echo(f"Dataset {dataset_slug} is created.")
    typer.echo(client.get_full_info_dataset(workspace, dataset_slug))

    return


@dataset_app.command()
def describe(
        dataset_arg: str = typer.Argument(...),
):
    """
    Describe the dataset information in detail
    """
    workspace, dataset, _ = parse_dataset_arg(dataset_arg)

    context = Context(login_required=True)
    client = SavviHubClient(token=context.token)

    dataset = client.dataset_read(workspace, dataset)
    if not dataset:
        typer.echo('[Error]Dataset not found!\n You should specify dataset with workspace. ex) savvihub/mnist')

    typer.echo(
        f'Name: {dataset.slug}\n'
        f'Volume ID: {dataset.main_volume_id}\n'
        f'Workspace:\n'
        f'\tName: {dataset.workspace.slug}'
    )

    source = dataset.source
    if source.type != DATASET_SOURCE_TYPE_SAVVIHUB:
        typer.echo(
            f'Source:\n'
            f'\tType: {source.type}\n'
            f'\tPath: {source.bucket_name}/{source.path}'
        )
        typer.echo(f'{client.get_full_info_dataset(dataset.workspace.slug, dataset.slug)}')



import collections
import numbers

import six

from savvihub import history
from savvihub.api.types import SavviHubExperiment


class Experiment(SavviHubExperiment):
    def __init__(self, experiment: SavviHubExperiment, client):
        super().__init__(experiment.dict)
        self.client = client
        self._history = None

    @classmethod
    def from_given(cls, experiment, client):
        experiment = cls(experiment, client)
        return experiment

    @property
    def history(self):
        if not self._history:
            self._history = history.History(self)
        return self._history

    def log(self, row, *, step=None):
        if not isinstance(row, collections.Mapping):
            raise ValueError(".log() takes a dictionary as a parameter")

        if any(not isinstance(key, six.string_types) for key in row.keys()):
            raise ValueError("The key of dictionary in .log() parameter must be str")

        for k in row.keys():
            if not k:
                raise ValueError("Logging empty key is not supported")

        if not isinstance(step, numbers.Number):
            raise ValueError(f"Step must be a number, not {type(step)}")
        if step < 0:
            raise ValueError(f"Step must be a positive integer, not {step}")
        if not isinstance(type(step), int):
            step = int(round(step))

        self.history.update(self.client, row, step)



from typer.testing import CliRunner

from savvihub.savvihub_cli import __version__
from savvihub.savvihub_cli import DEFAULT_CONFIG_PATH
from savvihub.common.utils import get_token_from_config

from savvihub.savvihub_cli.main import app

runner = CliRunner()


def test_version():
    result = runner.invoke(app, ["--version"])
    assert result.exit_code == 0
    assert f"SavviHub CLI Version: {__version__}" in result.stdout


def test_ping():
    result = runner.invoke(app, ["ping"])
    assert result.exit_code == 0
    assert "Response code: 200" in result.stdout
    assert "Response text: pong" in result.stdout


def test_init():
    token_input = "abcd"
    result = runner.invoke(app, ["init"], input=token_input)
    assert result.exit_code == 0
    assert f"Token successfully saved in {DEFAULT_CONFIG_PATH}" in result.stdout

    token_output = get_token_from_config(DEFAULT_CONFIG_PATH)
    assert token_input == token_output



import os
import sys

import typer

from savvihub import Context
from savvihub.api.savvihub import SavviHubClient
from savvihub.common.config_loader import ProjectConfigLoader
from savvihub.common.constants import WEB_HOST, API_HOST
from savvihub.common.git import GitRepository, InvalidGitRepository
from savvihub.common.utils import remove_file

project_app = typer.Typer()


@project_app.callback()
def main():
    """
    Create the basic element of workspace
    """
    return


@project_app.command()
def init(
    slug: str = typer.Argument(None, help="Type workspace/project as an argument"),
):
    """
    Initialize a new experiment configuration file with workspace/project
    """
    context = Context(login_required=True)
    if not context.user.github_authorized:
        typer.echo(f'You should authorize github first.\n{WEB_HOST}/cli/github-authorize/')
        return

    try:
        git_repo = GitRepository()
    except InvalidGitRepository as e:
        typer.echo(str(e))
        sys.exit(1)

    if not slug:
        workspaces = context.authorized_client.workspace_list(raise_error=True)
        default_workspace_slug = None
        if len(workspaces) == 1:
            default_workspace_slug = workspaces[0].slug

        workspace_slug = typer.prompt("SavviHub workspace name", default=default_workspace_slug)
        project_slug = typer.prompt("SavviHub project name", default=git_repo.get_github_repo()[1])
    else:
        workspace_slug, project_slug = slug.split("/")

    config_file_path = git_repo.get_savvihub_config_file_path()
    if os.path.exists(config_file_path):
        if typer.confirm(f'SavviHub config file already exists in {config_file_path}. Overwrite file?'):
            remove_file(config_file_path)
        else:
            return

    client = SavviHubClient(token=context.token)

    workspace = client.workspace_read(workspace_slug)
    if workspace is None:
        typer.echo(f'Cannot find workspace {workspace_slug}.')
        return

    project = client.project_read(workspace_slug, project_slug)
    if project is None:
        owner, repo = git_repo.get_github_repo()
        if typer.confirm(f'Project not found. Create new project in SavviHub?'):
            typer.echo(f'Workspace: {workspace_slug}\nProject: {project_slug}\nGit: github.com/{owner}/{repo}')
            client.project_github_create(workspace_slug, project_slug, owner, repo, raise_error=True)
            typer.echo(f'Project created successfully.\n{WEB_HOST}/{workspace_slug}/{project_slug}')

    project_config = ProjectConfigLoader(config_file_path)
    project_config.set_savvihub(url=API_HOST, workspace=workspace_slug, project=project_slug)
    project_config.save()

    typer.echo(f"Experiment config successfully made in {config_file_path}")


@project_app.command()
def status():
    """
    Print the current project configuration
    """
    context = Context(login_required=True, project_required=True)
    typer.echo(f'Workspace:{context.project.workspace.slug}\nProject: {context.project.slug}')



import re

from savvihub.common.constants import INQUIRER_NAME_IMAGE, INQUIRER_NAME_RESOURCE, INQUIRER_NAME_DATASET


def parse_title(choice):
    numbers = re.findall("[\s+]", choice)
    return numbers[0]


def parse_image_choice(image):
    return f'[{image.image_url}] {image.name}'


def parse_resource_choice(resource):
    return f'[{resource.name}] CPU: {resource.cpu_limit} | Memory: {resource.mem_limit}'


def parse_dataset_choice(dataset, dataset_type):
    return f'[{dataset.workspace.slug}/{dataset.slug}]'


def get_choices(client, req_type, workspace):
    if req_type == INQUIRER_NAME_IMAGE:
        savvi_kernel_images = client.kernel_image_list(workspace)
        choices = [None for _ in savvi_kernel_images]
        for (i, image) in enumerate(savvi_kernel_images):
            choices[i] = parse_image_choice(image)
        return choices
    elif req_type == INQUIRER_NAME_RESOURCE:
        savvi_kernel_resources = client.kernel_resource_list(workspace)
        choices = [None for _ in savvi_kernel_resources]
        for (i, resource) in enumerate(savvi_kernel_resources):
            choices[i] = parse_resource_choice(resource)
        return choices
    elif req_type == INQUIRER_NAME_DATASET:
        savvi_public_dataset = client.public_dataset_list(raise_error=True)
        savvi_private_dataset = client.dataset_list(workspace, raise_error=True)
        choices = []
        for dataset in savvi_public_dataset:
            choices.append(parse_dataset_choice(dataset, 'public'))
        for dataset in savvi_private_dataset:
            choices.append(parse_dataset_choice(dataset, 'private'))
        return choices
    else:
        raise Exception('Wrong type!')

